/*
 *	cutsrt Ver.0.01		Copyright (C) 2009  kt
 *	cutass Ver.0.01		(cutsrt mod)
 */

#include <ctype.h>
#include <stdio.h>
#include <vector>

std::vector<int> dellist;

void load_dellist(const char *filename);
double get_cut_frame_number(double frame, int mode);
void cut_ass(const char *file_in, const char *file_out);

double time2frame(double time);
double frame2time(double frame);

void usage()
{
	char *msg =
		"cutass Ver.0.01  (cutsrt mod  2011)\n"
		"\n"
		"usage:\n"
		"  cutass [<options>] <input.ass>\n"
		"\n"
		"options:\n"
		"  -l <AviUtl_DelList.txt>\tdefault: input_del.txt\n"
		"  -o <output.ass>\t\tdefault: input_new.ass\n"
	;
	fputs(msg, stderr);
}

int main(int argc, char *argv[])
{
	char buf[_MAX_PATH], buf2[_MAX_PATH];
	char *listfile = NULL, *infile = NULL, *outfile = NULL;
	
	while (*++argv) {
		char c = (*argv)[0];
		if ((c == '-') || (c == '/')) {
			switch (tolower((*argv)[1])) {
			case 'l':
				listfile = *++argv;
				break;
			case 'o':
				outfile = *++argv;
				break;
			case 'h':
				usage();
				break;
			}
			if (*argv == NULL) {
				usage();
				break;
			}
		} else {
			infile = *argv;
			if ((listfile == NULL) || (outfile == NULL)) {
				char drive[_MAX_DRIVE], dir[_MAX_DIR], fname[_MAX_FNAME];
				_splitpath(infile, drive, dir, fname, NULL);
				if (listfile == NULL) {
					listfile = buf;
					_makepath(listfile, drive, dir, fname, NULL);
					strcat(listfile, "_del.txt");
				}
				if (outfile == NULL) {
					outfile = buf2;
					_makepath(outfile, drive, dir, fname, NULL);
					strcat(outfile, "_new.ass");
				}
			}
			load_dellist(listfile);
			cut_ass(infile, outfile);
			listfile = NULL;
			outfile = NULL;
		}
	}
	if (infile == NULL) {
		usage();
	}
	return 0;
}

void load_dellist(const char *filename)
{
	dellist.clear();
	
	FILE *fp = fopen(filename, "rt");
	if (fp == NULL) {
		fprintf(stderr, "list file open error: %s\n", filename);
		exit(1);
	}
	char buf[256];
	if (fgets(buf, sizeof(buf), fp) == NULL) {
		fprintf(stderr, "list file read error\n");
		exit(1);
	}
	int start, end;
	if (sscanf(buf, "%d * %d", &start, &end) != 2) {
		fprintf(stderr, "list file format error\n");
		exit(1);
	}
	dellist.push_back(-1);			// sentinel
	dellist.push_back(start - 1);
	while (fgets(buf, sizeof(buf), fp) != NULL) {
		int del_start, del_end;
		sscanf(buf, "%d - %d", &del_start, &del_end);
		dellist.push_back(del_start);
		dellist.push_back(del_end);
	}
	dellist.push_back(end + 1);
	dellist.push_back(-1);			// sentinel
	dellist.push_back(-1);			// sentinel		mod fix(pre)
	fclose(fp);
}

double get_cut_frame_number(double frame, int mode)
{
	int cutframes = -1;
	for (int i = 0; i < dellist.size() - 1; i += 2) {
//		if (dellist[i+1] < 0) { break; }	// mod fix
		int oldcutframes = cutframes;
		cutframes += dellist[i+1] - dellist[i] + 1;
		if ((dellist[i] <= frame) && (frame <= dellist[i+1])) {
			// deleted
			if (mode > 0) {
				return dellist[i+1] + 1 - cutframes;
			} else if (mode < 0) {
				return dellist[i] - 1 - oldcutframes;
			} else {
				return -1;
			}
		} else if ((dellist[i+1] < frame) && (frame < dellist[i+2])) {
			// vaild
			return frame - cutframes;
		}
	}
	// not found
	if (mode < 0) {
		return dellist[dellist.size() - 1];
	}
	return -1;
}


double time2frame(double time)
{
	return time * 30 / 1001;	// 29.97 fps
}

double frame2time(double frame)
{
	return frame * 1001 / 30;	// 29.97 fps
}


void cut_ass(const char *file_in, const char *file_out)
{
	FILE *fpin = fopen(file_in, "rt");
	if (fpin == NULL) {
		fprintf(stderr, "input ass open error: %s\n", file_in);
		exit(1);
	}
	FILE *fpout = fopen(file_out, "wt");
	if (fpout == NULL) {
		fprintf(stderr, "output ass open error: %s\n", file_out);
		exit(1);
	}
	char buf[256];
	char *ass_str;
	while (fgets(buf, sizeof(buf), fpin) != NULL) {
		int sh, sm, ss, sms;
		int eh, em, es, ems;
		int Layer;
		if (sscanf(buf, "Dialogue: %d,%01d:%02d:%02d.%02d,%d:%02d:%02d.%02d,",
				&Layer, &sh, &sm, &ss, &sms, &eh, &em, &es, &ems) == 9) {
			ass_str = buf;
			for (int i=0;i<3;i++) {
				ass_str = strchr(ass_str,',');
				ass_str++;
			}
			int start = ((sh*60+sm)*60+ss)*1000+sms*10;
			int end   = ((eh*60+em)*60+es)*1000+ems*10;
			
			start = frame2time(get_cut_frame_number(time2frame(start), +1));
			end = frame2time(get_cut_frame_number(time2frame(end), -1));
			
			if (start >= end) {
				while (fgets(buf, sizeof(buf), fpin) != NULL) {
					if (buf[0] == '\n') {
						break;
					}
				}
				continue;
			}
			
			sms = start % 1000 / 10;
			start /= 1000;
			ss = start % 60;
			start /= 60;
			sm = start % 60;
			start /= 60;
			sh = start % 60;
			
			ems = end % 1000 / 10;
			end /= 1000;
			es = end % 60;
			end /= 60;
			em = end % 60;
			end /= 60;
			eh = end % 60;
			
			fprintf(fpout, "Dialogue: %d,%01d:%02d:%02d.%02d,%d:%02d:%02d.%02d,%s",
					Layer,
					sh, sm, ss, sms, eh, em, es, ems, ass_str);
		} else {
			fputs(buf, fpout);
		}
	}
	fclose(fpout);
	fclose(fpin);
}
